@@ -41,6 +41,9 @@
 #define UPOWER_DBUS_PATH                  "/org/freedesktop/UPower"
 #define UPOWER_DBUS_INTERFACE             "org.freedesktop.UPower"
 
+#define UPOWER_DBUS_DISPLAY_DEVICE_PATH   "/org/freedesktop/UPower/devices/DisplayDevice"
+#define UPOWER_DBUS_DEVICE_INTERFACE      "org.freedesktop.UPower.Device"
+
 #define LOGIND_DBUS_NAME                  "org.freedesktop.login1"
 #define LOGIND_DBUS_PATH                  "/org/freedesktop/login1"
 #define LOGIND_DBUS_INTERFACE             "org.freedesktop.login1.Manager"
@@ -55,6 +58,8 @@ typedef struct {
   GOptionGroup *group;
   GLogLevelFlags log_level;
   gboolean replace;
+  gboolean disable_upower;
+  gboolean disable_logind;
   GStrv blocked_drivers;
   GStrv blocked_actions;
 } DebugOptions;
@@ -81,15 +86,18 @@ typedef struct {
   GPtrArray *actions;
   GHashTable *profile_holds;
 
+  gboolean battery_support;
   GDBusProxy *upower_proxy;
+  GDBusProxy *upower_display_proxy;
   gulong upower_watch_id;
+  gulong upower_display_watch_id;
   gulong upower_properties_id;
+  gulong upower_display_properties_id;
   PpdPowerChangedReason power_changed_reason;
 
   guint logind_sleep_signal_id;
 
-  GStrv blocked_drivers;
-  GStrv blocked_actions;
+  DebugOptions *debug_options;
 } PpdApp;
 
 typedef struct {
@@ -130,6 +138,7 @@ static void upower_battery_set_power_changed_reason (PpdApp *, PpdPowerChangedRe
 /* profile drivers and actions */
 #include "ppd-action-trickle-charge.h"
 #include "ppd-action-amdgpu-panel-power.h"
+#include "ppd-action-amdgpu-dpm.h"
 #include "ppd-driver-placeholder.h"
 #include "ppd-driver-platform-profile.h"
 #include "ppd-driver-intel-pstate.h"
@@ -151,6 +160,7 @@ static GTypeGetFunc objects[] = {
   /* Actions */
   ppd_action_trickle_charge_get_type,
   ppd_action_amdgpu_panel_power_get_type,
+  ppd_action_amdgpu_dpm_get_type,
 };
 
 typedef enum {
@@ -161,6 +171,7 @@ typedef enum {
   PROP_DEGRADED                   = 1 << 4,
   PROP_ACTIVE_PROFILE_HOLDS       = 1 << 5,
   PROP_VERSION                    = 1 << 6,
+  PROP_UPOWER                     = 1 << 7,
 } PropertiesMask;
 
 #define PROP_ALL (PROP_ACTIVE_PROFILE       | \
@@ -169,7 +180,8 @@ typedef enum {
                   PROP_ACTIONS              | \
                   PROP_DEGRADED             | \
                   PROP_ACTIVE_PROFILE_HOLDS | \
-                  PROP_VERSION)
+                  PROP_VERSION              | \
+                  PROP_UPOWER)
 
 static gboolean
 driver_profile_support (PpdDriver *driver,
@@ -269,7 +281,7 @@ get_profiles_variant (PpdApp *data)
 }
 
 static GVariant *
-get_actions_variant (PpdApp *data)
+get_legacy_actions_variant (PpdApp *data)
 {
   GVariantBuilder builder;
   guint i;
@@ -279,12 +291,41 @@ get_actions_variant (PpdApp *data)
   for (i = 0; i < data->actions->len; i++) {
     PpdAction *action = g_ptr_array_index (data->actions, i);
 
+    if (!ppd_action_get_active (action))
+      continue;
+
     g_variant_builder_add (&builder, "s", ppd_action_get_action_name (action));
   }
 
   return g_variant_builder_end (&builder);
 }
 
+static GVariant *
+get_modern_actions_variant (PpdApp *data)
+{
+  GVariantBuilder builder;
+  guint i;
+
+  g_variant_builder_init (&builder, G_VARIANT_TYPE ("aa{sv}"));
+
+  for (i = 0; i < data->actions->len; i++) {
+    PpdAction *action = g_ptr_array_index (data->actions, i);
+    GVariantBuilder asv_builder;
+
+    g_variant_builder_init (&asv_builder, G_VARIANT_TYPE ("a{sv}"));
+    g_variant_builder_add (&asv_builder, "{sv}", "Name",
+                           g_variant_new_string (ppd_action_get_action_name (action)));
+    g_variant_builder_add (&asv_builder, "{sv}", "Description",
+                           g_variant_new_string (ppd_action_get_action_description (action)));
+    g_variant_builder_add (&asv_builder, "{sv}", "Enabled",
+                           g_variant_new_boolean (ppd_action_get_active (action)));
+    g_variant_builder_add (&builder, "a{sv}", &asv_builder);
+  }
+
+  return g_variant_builder_end (&builder);
+}
+
+
 static GVariant *
 get_profile_holds_variant (PpdApp *data)
 {
@@ -348,8 +389,12 @@ send_dbus_event_iface (PpdApp         *data,
                            get_profiles_variant (data));
   }
   if (mask & PROP_ACTIONS) {
-    g_variant_builder_add (&props_builder, "{sv}", "Actions",
-                           get_actions_variant (data));
+    if (g_str_equal (iface, POWER_PROFILES_IFACE_NAME))
+      g_variant_builder_add (&props_builder, "{sv}", "Actions",
+                             get_modern_actions_variant (data));
+    else
+      g_variant_builder_add (&props_builder, "{sv}", "Actions",
+                             get_legacy_actions_variant (data));
   }
   if (mask & PROP_ACTIVE_PROFILE_HOLDS) {
     g_variant_builder_add (&props_builder, "{sv}", "ActiveProfileHolds",
@@ -359,6 +404,10 @@ send_dbus_event_iface (PpdApp         *data,
     g_variant_builder_add (&props_builder, "{sv}", "Version",
                            g_variant_new_string (VERSION));
   }
+  if (mask & PROP_UPOWER) {
+    g_variant_builder_add (&props_builder, "{sv}", "BatteryAware",
+                           g_variant_new_boolean (data->battery_support));
+  }
 
   props_changed = g_variant_new ("(s@a{sv}@as)", iface,
                                  g_variant_builder_end (&props_builder),
@@ -402,6 +451,15 @@ save_configuration (PpdApp *data)
   g_key_file_set_string (data->config, "State", "Profile",
                          ppd_profile_to_str (data->active_profile));
 
+  for (guint i = 0; i < data->actions->len; i++) {
+    PpdAction *action = g_ptr_array_index (data->actions, i);
+    g_key_file_set_boolean (data->config, "Actions",
+                            ppd_action_get_action_name (action),
+                            ppd_action_get_active (action));
+  }
+
+  g_key_file_set_boolean (data->config, "State", "battery_aware", data->battery_support);
+
   if (!g_key_file_save_to_file (data->config, data->config_path, &error))
     g_warning ("Could not save configuration file '%s': %s", data->config_path, error->message);
 }
@@ -412,8 +470,17 @@ apply_configuration (PpdApp *data)
   g_autofree char *platform_driver = NULL;
   g_autofree char *profile_str = NULL;
   g_autofree char *cpu_driver = NULL;
+  g_autoptr(GError) error = NULL;
   PpdProfile profile;
 
+  data->battery_support = g_key_file_get_boolean (data->config, "State", "battery_aware", &error);
+  if (error != NULL) {
+    g_debug("battery_aware key not found: %s, defaulting to TRUE", error->message);
+    data->battery_support = TRUE;
+    g_key_file_set_boolean (data->config, "State", "battery_aware", TRUE);
+    g_clear_error (&error);
+  }
+
   cpu_driver = g_key_file_get_string (data->config, "State", "CpuDriver", NULL);
   if (PPD_IS_DRIVER_CPU (data->cpu_driver) &&
       g_strcmp0 (ppd_driver_get_driver_name (PPD_DRIVER (data->cpu_driver)), cpu_driver) != 0)
@@ -453,6 +520,8 @@ load_configuration (PpdApp *data)
   data->config = g_key_file_new ();
   if (!g_key_file_load_from_file (data->config, data->config_path, G_KEY_FILE_KEEP_COMMENTS, &error))
     g_debug ("Could not load configuration file '%s': %s", data->config_path, error->message);
+  else
+    g_debug ("Loaded configuration file '%s'", data->config_path);
 }
 
 static void
@@ -469,6 +538,9 @@ actions_activate_profile (GPtrArray *actions,
 
     action = g_ptr_array_index (actions, i);
 
+    if (!ppd_action_get_active (action))
+      continue;
+
     if (!ppd_action_activate_profile (action, profile, &error))
       g_warning ("Failed to activate action '%s' to profile %s: %s",
                  ppd_profile_to_str (profile),
@@ -569,6 +641,24 @@ release_all_profile_holds (PpdApp *data)
   g_hash_table_remove_all (data->profile_holds);
 }
 
+static gboolean
+set_battery_support (PpdApp *data, gboolean battery_support, GError **error)
+{
+  if (data->battery_support == battery_support) {
+    g_set_error (error, G_DBUS_ERROR, G_DBUS_ERROR_FAILED,
+                 "battery_aware is already set to %s", battery_support ? "TRUE" : "FALSE");
+    return FALSE;
+  }
+
+  data->battery_support = battery_support;
+  g_key_file_set_boolean (data->config, "State", "battery_aware", data->battery_support);
+  g_debug("battery_aware set to %s", data->battery_support ? "TRUE" : "FALSE");
+
+  restart_profile_drivers_for_default_app ();
+
+  return TRUE;
+}
+
 static gboolean
 set_active_profile (PpdApp      *data,
                     const char  *profile,
@@ -861,8 +951,13 @@ handle_get_property (GDBusConnection *connection,
     return g_variant_new_string ("");
   if (g_strcmp0 (property_name, "Profiles") == 0)
     return get_profiles_variant (data);
-  if (g_strcmp0 (property_name, "Actions") == 0)
-    return get_actions_variant (data);
+  if (g_str_equal (property_name, "ActionsInfo") &&
+      g_str_equal (interface_name, POWER_PROFILES_IFACE_NAME))
+      return get_modern_actions_variant (data);
+  if (g_str_equal (property_name, "Actions"))
+      return get_legacy_actions_variant (data);
+  if (g_str_equal (property_name, "BatteryAware"))
+      return g_variant_new_boolean (data->battery_support);
   if (g_strcmp0 (property_name, "PerformanceDegraded") == 0) {
     gchar *degraded = get_performance_degraded (data);
     return g_variant_new_take_string (g_steal_pointer (&degraded));
@@ -871,6 +966,8 @@ handle_get_property (GDBusConnection *connection,
     return get_profile_holds_variant (data);
   if (g_strcmp0 (property_name, "Version") == 0)
     return g_variant_new_string (VERSION);
+  g_set_error (error, G_DBUS_ERROR, G_DBUS_ERROR_FAILED,
+               "Invalid property'%s'", property_name);
   return NULL;
 }
 
@@ -885,22 +982,56 @@ handle_set_property (GDBusConnection  *connection,
                      gpointer          user_data)
 {
   PpdApp *data = user_data;
-  const char *profile;
 
   g_return_val_if_fail (data->connection, FALSE);
 
-  if (g_strcmp0 (property_name, "ActiveProfile") != 0) {
-    g_set_error (error, G_DBUS_ERROR, G_DBUS_ERROR_FAILED,
-                 "No such property: %s", property_name);
-    return FALSE;
-  }
-  if (!check_action_permission (data, sender,
-                                POWER_PROFILES_POLICY_NAMESPACE ".switch-profile",
-                                error))
-    return FALSE;
+  if (g_str_equal (property_name, "ActiveProfile")) {
+    const char *profile;
 
-  g_variant_get (value, "&s", &profile);
-  return set_active_profile (data, profile, error);
+    if (!check_action_permission (data, sender,
+                                  POWER_PROFILES_POLICY_NAMESPACE ".switch-profile",
+                                  error))
+      return FALSE;
+
+    g_variant_get (value, "&s", &profile);
+    return set_active_profile (data, profile, error);
+  } else if (g_str_equal (property_name, "BatteryAware")) {
+    if (!check_action_permission (data,
+                                  sender,
+                                  POWER_PROFILES_POLICY_NAMESPACE ".configure-battery-aware",
+                                  error)) {
+      return FALSE;
+    }
+
+    return set_battery_support (data, g_variant_get_boolean (value), error);
+  }
+
+  g_set_error (error, G_DBUS_ERROR, G_DBUS_ERROR_FAILED,
+               "No such property: %s", property_name);
+  return FALSE;
+}
+
+static gboolean set_action_enabled (PpdApp                *data,
+                                    GVariant              *parameters,
+                                    GError                **error)
+{
+  const char *action_name;
+  gboolean active;
+
+  g_variant_get (parameters, "(&sb)", &action_name, &active);
+
+  for (guint i = 0; i < data->actions->len; i++) {
+    PpdAction *action = g_ptr_array_index (data->actions, i);
+
+    if (g_strcmp0 (ppd_action_get_action_name (action), action_name) == 0) {
+      g_key_file_set_boolean (data->config, "Actions", action_name, active);
+      restart_profile_drivers_for_default_app ();
+      return TRUE;
+    }
+  }
+  g_set_error (error, G_DBUS_ERROR, G_DBUS_ERROR_INVALID_ARGS,
+               "No such action '%s'", action_name);
+  return FALSE;
 }
 
 static void
@@ -935,6 +1066,29 @@ handle_method_call (GDBusConnection       *connection,
     hold_profile (data, parameters, invocation);
   } else if (g_strcmp0 (method_name, "ReleaseProfile") == 0) {
     release_profile (data, parameters, invocation);
+  } else if (g_strcmp0 (method_name, "SetActionEnabled") == 0) {
+    g_autoptr(GError) local_error = NULL;
+
+    if (g_str_equal (interface_name, POWER_PROFILES_LEGACY_IFACE_NAME)) {
+      g_dbus_method_invocation_return_error (invocation, G_DBUS_ERROR, G_DBUS_ERROR_UNKNOWN_METHOD,
+                                             "Method %s is not available in interface %s", method_name,
+                                             interface_name);
+      return;
+    }
+
+    if (!check_action_permission (data,
+                                  g_dbus_method_invocation_get_sender (invocation),
+                                  POWER_PROFILES_POLICY_NAMESPACE ".configure-action",
+                                  &local_error)) {
+      g_dbus_method_invocation_return_gerror (invocation, local_error);
+      return;
+    }
+
+    if (!set_action_enabled (data, parameters, &local_error)) {
+      g_dbus_method_invocation_return_gerror (invocation, local_error);
+      return;
+    }
+    g_dbus_method_invocation_return_value (invocation, NULL);
   } else {
       g_dbus_method_invocation_return_error (invocation, G_DBUS_ERROR, G_DBUS_ERROR_UNKNOWN_METHOD,
                                              "No such method %s in interface %s", interface_name,
@@ -1023,8 +1177,8 @@ bus_acquired_handler (GDBusConnection *connection,
 }
 
 static void
-upower_battery_update_state_from_value (PpdApp   *data,
-                                        GVariant *battery_val)
+upower_source_update_from_value (PpdApp   *data,
+                                 GVariant *battery_val)
 {
   PpdPowerChangedReason reason;
 
@@ -1039,12 +1193,66 @@ upower_battery_update_state_from_value (PpdApp   *data,
 }
 
 static void
-upower_battery_update_state (PpdApp *data)
+upower_source_update (PpdApp *data)
 {
   g_autoptr(GVariant) battery_val = NULL;
 
   battery_val = g_dbus_proxy_get_cached_property (data->upower_proxy, "OnBattery");
-  upower_battery_update_state_from_value (data, battery_val);
+  upower_source_update_from_value (data, battery_val);
+}
+
+static void
+upower_battery_changed (PpdApp *data, gdouble level)
+{
+  g_info ("Battery level changed to %f", level);
+
+  for (guint i = 0; i < data->actions->len; i++) {
+    g_autoptr(GError) error = NULL;
+    PpdAction *action;
+
+    action = g_ptr_array_index (data->actions, i);
+
+    if (!ppd_action_get_active (action))
+      continue;
+
+    if (!ppd_action_battery_changed (action, level, &error)) {
+      g_warning ("failed to update action %s: %s",
+                 ppd_action_get_action_name (action),
+                 error->message);
+      g_clear_error (&error);
+      continue;
+    }
+  }
+
+  if (PPD_IS_DRIVER_CPU (data->cpu_driver)) {
+    g_autoptr(GError) error = NULL;
+
+    if (!ppd_driver_battery_changed (PPD_DRIVER (data->cpu_driver), level, &error)) {
+      g_warning ("failed to update driver %s: %s",
+                 ppd_driver_get_driver_name (PPD_DRIVER (data->cpu_driver)),
+                 error->message);
+    }
+  }
+
+  if (PPD_IS_DRIVER_PLATFORM (data->platform_driver)) {
+    g_autoptr(GError) error = NULL;
+
+    if (!ppd_driver_battery_changed (PPD_DRIVER (data->platform_driver), level, &error)) {
+      g_warning ("failed to update driver %s: %s",
+                 ppd_driver_get_driver_name (PPD_DRIVER (data->platform_driver)),
+                 error->message);
+    }
+  }
+}
+
+static void
+upower_battery_update (PpdApp *data)
+{
+  g_autoptr(GVariant) val = NULL;
+
+  val = g_dbus_proxy_get_cached_property (data->upower_display_proxy, "Percentage");
+  if (val)
+    upower_battery_changed(data, g_variant_get_double (val));
 }
 
 static void
@@ -1055,12 +1263,18 @@ upower_properties_changed (GDBusProxy *proxy,
 {
   g_auto(GVariantDict) props_dict = G_VARIANT_DICT_INIT (changed_properties);
   g_autoptr(GVariant) battery_val = NULL;
+  g_autoptr(GVariant) percent_val = NULL;
 
   battery_val = g_variant_dict_lookup_value (&props_dict, "OnBattery",
                                              G_VARIANT_TYPE_BOOLEAN);
 
   if (battery_val)
-    upower_battery_update_state_from_value (data, battery_val);
+    upower_source_update_from_value (data, battery_val);
+
+  percent_val = g_variant_dict_lookup_value (&props_dict, "Percentage",
+                                             G_VARIANT_TYPE_DOUBLE);
+  if (percent_val)
+    upower_battery_changed (data, g_variant_get_double (percent_val));
 }
 
 static void
@@ -1080,6 +1294,9 @@ upower_battery_set_power_changed_reason (PpdApp                *data,
 
     action = g_ptr_array_index (data->actions, i);
 
+    if (!ppd_action_get_active (action))
+      continue;
+
     if (!ppd_action_power_changed (action, reason, &error)) {
       g_warning ("failed to update action %s: %s",
                  ppd_action_get_action_name (action),
@@ -1123,7 +1340,7 @@ upower_name_owner_changed (GObject    *object,
 
   if (name_owner != NULL) {
     g_debug ("%s appeared", UPOWER_DBUS_NAME);
-    upower_battery_update_state (data);
+    upower_source_update (data);
     return;
   }
 
@@ -1158,12 +1375,51 @@ on_upower_proxy_cb (GObject *source_object,
                                                  G_CALLBACK (upower_properties_changed),
                                                  data);
 
-  data->upower_watch_id = g_signal_connect (data->upower_proxy,
-                                            "notify::g-name-owner",
-                                            G_CALLBACK (upower_name_owner_changed),
-                                            data);
+  if (!data->upower_display_watch_id) {
+    data->upower_watch_id = g_signal_connect (data->upower_proxy,
+                                              "notify::g-name-owner",
+                                              G_CALLBACK (upower_name_owner_changed),
+                                              data);
+  }
 
-  upower_battery_update_state (data);
+  upower_source_update (data);
+}
+
+static void
+on_upower_display_proxy_cb (GObject *source_object,
+                            GAsyncResult *res,
+                            gpointer user_data)
+{
+  PpdApp *data = user_data;
+  g_autoptr(GDBusProxy) proxy = NULL;
+  g_autoptr(GError) error = NULL;
+
+  proxy = g_dbus_proxy_new_finish (res, &error);
+
+  if (proxy == NULL) {
+    if (g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
+      return;
+
+    g_warning ("failed to connect to upower: %s", error->message);
+    return;
+  }
+
+  g_return_if_fail (data->upower_display_proxy == NULL);
+  data->upower_display_proxy = g_steal_pointer (&proxy);
+
+  data->upower_display_properties_id = g_signal_connect (data->upower_display_proxy,
+                                                         "g-properties-changed",
+                                                         G_CALLBACK (upower_properties_changed),
+                                                         data);
+
+  if (!data->upower_watch_id) {
+    data->upower_display_watch_id = g_signal_connect (data->upower_display_proxy,
+                                                      "notify::g-name-owner",
+                                                      G_CALLBACK (upower_name_owner_changed),
+                                                      data);
+  }
+
+  upower_battery_update (data);
 }
 
 static void
@@ -1273,9 +1529,13 @@ stop_profile_drivers (PpdApp *data)
   g_ptr_array_set_size (data->actions, 0);
   g_clear_signal_handler (&data->upower_watch_id, data->upower_proxy);
   g_clear_signal_handler (&data->upower_properties_id, data->upower_proxy);
+  g_clear_signal_handler (&data->upower_display_watch_id, data->upower_display_proxy);
+  g_clear_signal_handler (&data->upower_display_properties_id, data->upower_display_proxy);
   g_clear_object (&data->cancellable);
   maybe_disconnect_object_by_data (data->upower_proxy, data);
   g_clear_object (&data->upower_proxy);
+  maybe_disconnect_object_by_data (data->upower_display_proxy, data);
+  g_clear_object (&data->upower_display_proxy);
   maybe_disconnect_object_by_data (data->cpu_driver, data);
   g_clear_object (&data->cpu_driver);
   maybe_disconnect_object_by_data (data->platform_driver, data);
@@ -1286,15 +1546,30 @@ static gboolean
 action_blocked (PpdApp *app, PpdAction *action)
 {
   const gchar *action_name = ppd_action_get_action_name (action);
+  g_autoptr(GError) error = NULL;
   gboolean blocked;
 
-  if (app->blocked_actions == NULL || g_strv_length (app->blocked_actions) == 0)
-    return FALSE;
+  if (app->debug_options->blocked_actions != NULL &&
+      g_strv_length (app->debug_options->blocked_actions) != 0) {
 
-  blocked = g_strv_contains ((const gchar *const *) app->blocked_actions, action_name);
+    blocked = g_strv_contains ((const gchar *const *) app->debug_options->blocked_actions, action_name);
 
-  if (blocked)
-    g_debug ("Action '%s' is blocked", action_name);
+    if (blocked) {
+      g_debug ("Action '%s' is blocked by command line", action_name);
+      return TRUE;
+    }
+  }
+
+  blocked = !g_key_file_get_boolean (app->config, "Actions", action_name, &error);
+
+  /* if not in conffile, fallback to action opt-in */
+  if (error != NULL) {
+    blocked = ppd_action_get_optin (action);
+    g_debug ("Action '%s' is set %s by default", action_name, blocked ? "disabled" : "enabled");
+    return blocked;
+  }
+
+  g_debug ("Action '%s' is %s by configuration", action_name, blocked ? "disabled" : "enabled");
   return blocked;
 }
 
@@ -1304,10 +1579,10 @@ driver_blocked (PpdApp *app, PpdDriver *driver)
   const gchar *driver_name = ppd_driver_get_driver_name (driver);
   gboolean blocked;
 
-  if (app->blocked_drivers == NULL || g_strv_length (app->blocked_drivers) == 0)
+  if (app->debug_options->blocked_drivers == NULL || g_strv_length (app->debug_options->blocked_drivers) == 0)
     return FALSE;
 
-  blocked = g_strv_contains ((const gchar *const *) app->blocked_drivers, driver_name);
+  blocked = g_strv_contains ((const gchar *const *) app->debug_options->blocked_drivers, driver_name);
   if (blocked)
     g_debug ("Driver '%s' is blocked", driver_name);
   return blocked;
@@ -1318,7 +1593,8 @@ start_profile_drivers (PpdApp *data)
 {
   guint i;
   g_autoptr(GError) initial_error = NULL;
-  gboolean needs_battery_monitor = FALSE;
+  gboolean needs_battery_state_monitor = FALSE;
+  gboolean needs_battery_change_monitor = FALSE;
   gboolean needs_suspend_monitor = FALSE;
 
   data->cancellable = g_cancellable_new ();
@@ -1383,10 +1659,14 @@ start_profile_drivers (PpdApp *data)
         g_return_if_reached ();
 
       if (PPD_DRIVER_GET_CLASS (driver)->power_changed != NULL)
-        needs_battery_monitor = TRUE;
+        needs_battery_state_monitor = TRUE;
+
+      if (PPD_DRIVER_GET_CLASS (driver)->battery_changed != NULL)
+        needs_battery_change_monitor = TRUE;
 
       if (PPD_DRIVER_GET_CLASS (driver)->prepare_to_sleep != NULL)
         needs_suspend_monitor = TRUE;
+
       g_info ("Driver '%s' loaded", ppd_driver_get_driver_name (driver));
 
       g_signal_connect (G_OBJECT (driver), "notify::performance-degraded",
@@ -1402,20 +1682,27 @@ start_profile_drivers (PpdApp *data)
       g_debug ("Handling action '%s'", ppd_action_get_action_name (action));
 
       if (action_blocked (data, action)) {
-        g_debug ("Action '%s' is blocked, skipping", ppd_action_get_action_name (action));
-        continue;
-      }
-
-      if (ppd_action_probe (action) == PPD_PROBE_RESULT_FAIL) {
-        g_debug ("probe () failed for action '%s', skipping",
-                 ppd_action_get_action_name (action));
-        continue;
+          ppd_action_set_active (action, FALSE);
+      } else {
+        switch (ppd_action_probe(action)) {
+        case PPD_PROBE_RESULT_SUCCESS:
+          ppd_action_set_active (action, TRUE);
+          break;
+        default:
+          ppd_action_set_active (action, FALSE);
+          break;
+        }
       }
 
       if (PPD_ACTION_GET_CLASS (action)->power_changed != NULL)
-        needs_battery_monitor = TRUE;
+        needs_battery_state_monitor = TRUE;
 
-      g_info ("Action '%s' loaded", ppd_action_get_action_name (action));
+      if (PPD_ACTION_GET_CLASS (action)->battery_changed != NULL)
+        needs_battery_change_monitor = TRUE;
+
+      g_info ("Action '%s' active %d",
+              ppd_action_get_action_name (action),
+              ppd_action_get_active (action));
       g_ptr_array_add (data->actions, g_steal_pointer (&action));
       continue;
     }
@@ -1437,24 +1724,47 @@ start_profile_drivers (PpdApp *data)
   send_dbus_event (data, PROP_ALL);
   data->was_started = TRUE;
 
-  if (needs_battery_monitor) {
+  if (data->debug_options->disable_upower)
+    data->battery_support = FALSE;
+
+  if (!data->battery_support) {
+    g_debug ("upower is disabled, let's skip it");
+  } else if (needs_battery_state_monitor || needs_battery_change_monitor) {
     /* start watching for power changes */
-    g_debug ("Battery state monitor required, connecting to upower...");
-    g_dbus_proxy_new (data->connection,
-                      G_DBUS_PROXY_FLAGS_DO_NOT_AUTO_START |
-                      G_DBUS_PROXY_FLAGS_DO_NOT_CONNECT_SIGNALS,
-                      NULL,
-                      UPOWER_DBUS_NAME,
-                      UPOWER_DBUS_PATH,
-                      UPOWER_DBUS_INTERFACE,
-                      data->cancellable,
-                      on_upower_proxy_cb,
-                      data);
+    if (needs_battery_state_monitor) {
+      g_debug ("Battery state monitor required, connecting to upower...");
+      g_dbus_proxy_new (data->connection,
+                        G_DBUS_PROXY_FLAGS_DO_NOT_AUTO_START |
+                        G_DBUS_PROXY_FLAGS_DO_NOT_CONNECT_SIGNALS,
+                        NULL,
+                        UPOWER_DBUS_NAME,
+                        UPOWER_DBUS_PATH,
+                        UPOWER_DBUS_INTERFACE,
+                        data->cancellable,
+                        on_upower_proxy_cb,
+                        data);
+    }
+
+    if (needs_battery_change_monitor) {
+      g_debug ("Battery change monitor required, connecting to upower...");
+      g_dbus_proxy_new (data->connection,
+                        G_DBUS_PROXY_FLAGS_DO_NOT_AUTO_START |
+                        G_DBUS_PROXY_FLAGS_DO_NOT_CONNECT_SIGNALS,
+                        NULL,
+                        UPOWER_DBUS_NAME,
+                        UPOWER_DBUS_DISPLAY_DEVICE_PATH,
+                        UPOWER_DBUS_DEVICE_INTERFACE,
+                        data->cancellable,
+                        on_upower_display_proxy_cb,
+                        data);
+    }
   } else {
     g_debug ("No battery state monitor required by any driver, let's skip it");
   }
 
-  if (needs_suspend_monitor) {
+  if (data->debug_options->disable_logind) {
+    g_debug ("logind is disabled, let's skip it");
+  } else if (needs_suspend_monitor) {
     g_debug ("Suspension state monitor required, monitoring logind...");
     data->logind_sleep_signal_id =
       g_dbus_connection_signal_subscribe (data->connection,
@@ -1493,7 +1803,6 @@ name_acquired_handler (GDBusConnection *connection,
 
 static gboolean
 setup_dbus (PpdApp    *data,
-            gboolean   replace,
             GError   **error)
 {
   g_autoptr(GBytes) iface_data = NULL;
@@ -1532,7 +1841,7 @@ setup_dbus (PpdApp    *data,
   own_data->legacy_interface = g_dbus_interface_info_ref (legacy_introspection_data->interfaces[0]);
 
   own_data->flags = G_BUS_NAME_OWNER_FLAGS_ALLOW_REPLACEMENT;
-  if (replace)
+  if (data->debug_options->replace)
     own_data->flags |= G_BUS_NAME_OWNER_FLAGS_REPLACE;
 
   data->name_id = g_bus_own_name (G_BUS_TYPE_SYSTEM,
@@ -1558,8 +1867,7 @@ free_app_data (PpdApp *data)
   g_clear_handle_id (&data->name_id, g_bus_unown_name);
   g_clear_handle_id (&data->legacy_name_id, g_bus_unown_name);
 
-  g_strfreev (data->blocked_drivers);
-  g_strfreev (data->blocked_actions);
+  g_clear_pointer (&data->debug_options, debug_options_free);
   g_clear_pointer (&data->config_path, g_free);
   g_clear_pointer (&data->config, g_key_file_unref);
   g_clear_pointer (&data->probed_drivers, g_ptr_array_unref);
@@ -1708,6 +2016,24 @@ debug_pre_parse_hook (GOptionContext *context,
       "Block action(s) from loading",
       NULL,
     },
+    {
+      "disable-upower",
+      0,
+      G_OPTION_FLAG_NONE,
+      G_OPTION_ARG_NONE,
+      &data->disable_upower,
+      "Disable upower integration",
+      NULL,
+    },
+    {
+      "disable-logind",
+      0,
+      G_OPTION_FLAG_NONE,
+      G_OPTION_ARG_NONE,
+      &data->disable_logind,
+      "Disable logind integration",
+      NULL,
+    },
     { NULL }
   };
   g_option_group_add_entries (group, options);
@@ -1762,8 +2088,7 @@ int main (int argc, char **argv)
   data->profile_holds = g_hash_table_new_full (g_direct_hash, g_direct_equal, NULL, (GDestroyNotify) profile_hold_free);
   data->active_profile = PPD_PROFILE_BALANCED;
   data->selected_profile = PPD_PROFILE_BALANCED;
-  data->blocked_drivers = g_steal_pointer (&debug_options->blocked_drivers);
-  data->blocked_actions = g_steal_pointer (&debug_options->blocked_actions);
+  data->debug_options = g_steal_pointer(&debug_options);
 
   g_unix_signal_add (SIGTERM, quit_signal_callback, data);
   g_unix_signal_add (SIGINT, quit_signal_callback, data);
@@ -1774,7 +2099,7 @@ int main (int argc, char **argv)
   ppd_app = data;
 
   /* Set up D-Bus */
-  if (!setup_dbus (data, debug_options->replace, &error)) {
+  if (!setup_dbus (data, &error)) {
     g_error ("Failed to start dbus: %s", error->message);
     return EXIT_FAILURE;
   }
