@@ -17,20 +17,6 @@
 
 #define CPUFREQ_POLICY_DIR "/sys/devices/system/cpu/cpufreq/"
 #define PSTATE_STATUS_PATH "/sys/devices/system/cpu/amd_pstate/status"
-#define ACPI_PM_PROFILE "/sys/firmware/acpi/pm_profile"
-
-enum acpi_preferred_pm_profiles {
-  PM_UNSPECIFIED = 0,
-  PM_DESKTOP = 1,
-  PM_MOBILE = 2,
-  PM_WORKSTATION = 3,
-  PM_ENTERPRISE_SERVER = 4,
-  PM_SOHO_SERVER = 5,
-  PM_APPLIANCE_PC = 6,
-  PM_PERFORMANCE_SERVER = 7,
-  PM_TABLET = 8,
-  NR_PM_PROFILES = 9
-};
 
 struct _PpdDriverAmdPstate
 {
@@ -72,21 +58,12 @@ probe_epp (PpdDriverAmdPstate *pstate)
   g_autoptr(GDir) dir = NULL;
   g_autofree char *policy_dir = NULL;
   g_autofree char *pstate_status_path = NULL;
-  g_autofree char *status = NULL;
-  g_autofree char *pm_profile_path = NULL;
-  g_autofree char *pm_profile_str = NULL;
-  guint64 pm_profile;
   const char *dirname;
 
   /* Verify that AMD P-State is running in active mode */
   pstate_status_path = ppd_utils_get_sysfs_path (PSTATE_STATUS_PATH);
-  if (!g_file_get_contents (pstate_status_path, &status, NULL, NULL))
+  if (!g_file_test (pstate_status_path, G_FILE_TEST_EXISTS))
     return PPD_PROBE_RESULT_FAIL;
-  status = g_strchomp (status);
-  if (g_strcmp0 (status, "active") != 0) {
-    g_debug ("AMD P-State is not running in active mode");
-    return PPD_PROBE_RESULT_FAIL;
-  }
 
   policy_dir = ppd_utils_get_sysfs_path (CPUFREQ_POLICY_DIR);
   dir = g_dir_open (policy_dir, 0, NULL);
@@ -95,25 +72,10 @@ probe_epp (PpdDriverAmdPstate *pstate)
     return PPD_PROBE_RESULT_FAIL;
   }
 
-  /* only run on things that we know aren't servers */
-  pm_profile_path = ppd_utils_get_sysfs_path (ACPI_PM_PROFILE);
-  if (!g_file_get_contents (pm_profile_path, &pm_profile_str, NULL, NULL))
-    return PPD_PROBE_RESULT_FAIL;
-  pm_profile = g_ascii_strtoull (pm_profile_str, NULL, 10);
-  switch (pm_profile) {
-  case PM_UNSPECIFIED:
-  case PM_ENTERPRISE_SERVER:
-  case PM_SOHO_SERVER:
-  case PM_PERFORMANCE_SERVER:
-    g_debug ("AMD-P-State not supported on PM profile %" G_GUINT64_FORMAT, pm_profile);
-    return PPD_PROBE_RESULT_FAIL;
-  default:
-    break;
-  }
-
   while ((dirname = g_dir_read_name (dir)) != NULL) {
     g_autofree char *base = NULL;
     g_autofree char *path = NULL;
+    g_autofree char *contents = NULL;
     g_autoptr(GError) error = NULL;
 
     base = g_build_filename (policy_dir,
@@ -126,6 +88,15 @@ probe_epp (PpdDriverAmdPstate *pstate)
     if (!g_file_test (path, G_FILE_TEST_EXISTS))
       continue;
 
+    if (!g_file_get_contents (path, &contents, NULL, &error)) {
+      g_debug ("Failed to read %s: %s", path, error->message);
+      continue;
+    }
+    if (!ppd_utils_write (path, g_strchomp (contents), &error)) {
+      g_debug ("Failed to write %s: %s", path, error->message);
+      continue;
+    }
+
     if (!pstate->epp_devices)
       pstate->epp_devices = g_ptr_array_new_with_free_func (g_free);
 
@@ -183,6 +154,36 @@ profile_to_epp_pref (PpdProfile profile, gboolean battery)
   g_return_val_if_reached (NULL);
 }
 
+static const char *
+profile_to_cpb_pref (PpdProfile profile)
+{
+  switch (profile) {
+  case PPD_PROFILE_POWER_SAVER:
+    return "0";
+  case PPD_PROFILE_BALANCED:
+  case PPD_PROFILE_PERFORMANCE:
+    return "1";
+  }
+
+  g_return_val_if_reached (NULL);
+
+}
+
+static const char *
+profile_to_min_freq (PpdProfile profile)
+{
+  switch (profile) {
+  case PPD_PROFILE_POWER_SAVER:
+    return "cpuinfo_min_freq";
+  case PPD_PROFILE_BALANCED:
+  case PPD_PROFILE_PERFORMANCE:
+    return "amd_pstate_lowest_nonlinear_freq";
+  }
+
+  g_return_val_if_reached (NULL);
+
+}
+
 static gboolean
 apply_pref_to_devices (GPtrArray   *devices,
                        PpdProfile   profile,
@@ -191,17 +192,34 @@ apply_pref_to_devices (GPtrArray   *devices,
 {
   const char *epp_pref;
   const char *gov_pref;
+  const char *cpb_pref;
+  const char *min_freq;
+  g_autofree char *status = NULL;
+  g_autofree char *pstate_status_path = NULL;
 
   if (profile == PPD_PROFILE_UNSET)
     return TRUE;
 
+  pstate_status_path = ppd_utils_get_sysfs_path (PSTATE_STATUS_PATH);
+  if (!g_file_get_contents (pstate_status_path, &status, NULL, error))
+    return FALSE;
+  status = g_strchomp (status);
+  if (g_strcmp0 (status, "active") != 0) {
+    g_warning ("AMD P-State is not in active mode");
+    return TRUE;
+  }
+
   epp_pref = profile_to_epp_pref (profile, battery);
   gov_pref = profile_to_gov_pref (profile);
+  cpb_pref = profile_to_cpb_pref (profile);
+  min_freq = profile_to_min_freq (profile);
 
   for (guint i = 0; i < devices->len; ++i) {
     const char *base = g_ptr_array_index (devices, i);
     g_autofree char *epp = NULL;
     g_autofree char *gov = NULL;
+    g_autofree char *cpb = NULL;
+    g_autofree char *min_freq_path = NULL;
 
     gov = g_build_filename (base,
                             "scaling_governor",
@@ -216,6 +234,26 @@ apply_pref_to_devices (GPtrArray   *devices,
 
     if (!ppd_utils_write (epp, epp_pref, error))
       return FALSE;
+
+    cpb = g_build_filename (base, "boost", NULL);
+    if (g_file_test (cpb, G_FILE_TEST_EXISTS)) {
+      if (!ppd_utils_write (cpb, cpb_pref, error))
+        return FALSE;
+    }
+
+    min_freq_path = g_build_filename (base, min_freq, NULL);
+    if (g_file_test (min_freq_path, G_FILE_TEST_EXISTS)) {
+      g_autofree char *scaling_freq_path = NULL;
+      g_autofree char *min_freq_val = NULL;
+
+      if (!g_file_get_contents (min_freq_path, &min_freq_val, NULL, error))
+        return FALSE;
+      min_freq_val = g_strchomp (min_freq_val);
+
+      scaling_freq_path = g_build_filename (base, "scaling_min_freq", NULL);
+      if (!ppd_utils_write (scaling_freq_path, min_freq_val, error))
+        return FALSE;
+    }
   }
 
   return TRUE;
