@@ -13,31 +13,37 @@
 const char *
 ppd_profile_to_str (PpdProfile profile)
 {
-  GFlagsClass *klass = g_type_class_ref (PPD_TYPE_PROFILE);
+  g_autoptr(GFlagsClass) klass = g_type_class_ref (PPD_TYPE_PROFILE);
   GFlagsValue *value = g_flags_get_first_value (klass, profile);
   const gchar *name = value ? value->value_nick : "";
-  g_type_class_unref (klass);
   return name;
 }
 
 PpdProfile
 ppd_profile_from_str (const char *str)
 {
-  GFlagsClass *klass = g_type_class_ref (PPD_TYPE_PROFILE);
+  g_autoptr(GFlagsClass) klass = g_type_class_ref (PPD_TYPE_PROFILE);
   GFlagsValue *value = g_flags_get_value_by_nick (klass, str);
   PpdProfile profile = value ? value->value : PPD_PROFILE_UNSET;
-  g_type_class_unref (klass);
   return profile;
 }
 
 gboolean
 ppd_profile_has_single_flag (PpdProfile profile)
 {
-  GFlagsClass *klass = g_type_class_ref (PPD_TYPE_PROFILE);
+  g_autoptr(GFlagsClass) klass = g_type_class_ref (PPD_TYPE_PROFILE);
   GFlagsValue *value = g_flags_get_first_value (klass, profile);
-  gboolean ret = FALSE;
   if (value && value->value == profile)
-    ret = TRUE;
-  g_type_class_unref (klass);
-  return ret;
+    return TRUE;
+
+  return FALSE;
+}
+
+const char *
+ppd_power_changed_reason_to_str (PpdPowerChangedReason reason)
+{
+  g_autoptr(GEnumClass) klass = g_type_class_ref (PPD_TYPE_POWER_CHANGED_REASON);
+  GEnumValue *value = g_enum_get_value (klass, reason);
+  const gchar *name = value ? value->value_nick : "";
+  return name;
 }
